@@ -610,6 +610,20 @@ static bool validate_close_editor(WBMainWindow *mainWindow)
 
 //--------------------------------------------------------------------------------------------------
 
+static void call_toggle_fullscreen(WBMainWindow *mainWindow)
+{
+  [mainWindow.window toggleFullScreen: mainWindow.window];
+}
+
+//--------------------------------------------------------------------------------------------------
+
+static bool validate_toggle_fullscreen(WBMainWindow *mainWindow)
+{
+  return NSAppKitVersionNumber > NSAppKitVersionNumber10_6;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 static bool validate_find_replace()
 {
   id firstResponder = [[NSApp keyWindow] firstResponder];
@@ -772,9 +786,12 @@ static bool validate_redo(WBMainWindow *mainWindow)
                                                boost::bind(validate_closetab_old, mainWindow));
   _wbui->get_command_ui()->add_builtin_command("close_tab", boost::bind(call_close_tab, mainWindow), 
                                                boost::bind(validate_close_tab, mainWindow));
-  _wbui->get_command_ui()->add_builtin_command("close_editor", boost::bind(call_close_editor, mainWindow), 
+  _wbui->get_command_ui()->add_builtin_command("close_editor", boost::bind(call_close_editor, mainWindow),
                                                boost::bind(validate_close_editor, mainWindow));
 
+  _wbui->get_command_ui()->add_builtin_command("toggle_fullscreen", boost::bind(call_toggle_fullscreen, mainWindow),
+                                               boost::bind(validate_toggle_fullscreen, mainWindow));
+
   _wbui->get_command_ui()->add_builtin_command("find", boost::bind(call_find, mainWindow),
                                                boost::bind(validate_find, mainWindow));
   _wbui->get_command_ui()->add_builtin_command("find_replace", boost::bind(call_find_replace, true),
