@@ -1080,8 +1080,10 @@ void setup_mforms_app(WBMainWindow *mwin);
 - (void)awakeFromNib
 {
   [tabSwitcher setTabStyle: MMainTabSwitcher];
-  
-  [statusBarText setStringValue:@""];
+  if (NSAppKitVersionNumber > NSAppKitVersionNumber10_6)
+    [self.window setCollectionBehavior: NSWindowCollectionBehaviorFullScreenPrimary];
+
+  [statusBarText setStringValue: @""];
 }
 
 //--------------------------------------------------------------------------------------------------
