@@ -1423,4 +1423,105 @@ void EolHelpers::fix(const std::string &src_text, std::string &dest_text, Eol_fo
 
 //--------------------------------------------------------------------------------------------------
 
-} // namespace base
\ No newline at end of file
+std::string reflow_text(const std::string &text, unsigned int line_length, const std::string &left_fill, bool indent_first, unsigned int max_lines)
+{
+  bool use_fill = true;
+  const unsigned int minimum_text_length = 5;
+
+  //  Check if the line length complies to the minimum required
+  if (line_length < minimum_text_length)
+    return "";
+
+  //  Only use left_fill when it's small enough to fit in the line and make the function able
+  //  to do what it has to do
+  const unsigned int left_fill_length = left_fill.size();
+  
+  if (left_fill_length + minimum_text_length >= line_length)
+    use_fill = false;
+
+  //  Check for empty string...if we let it go, a left_fill will be inserted
+  if (text.size() == 0)
+    return "";
+  
+  //  Check if it's a valid utf8 string
+  const char *invalid_data_ptr = NULL;
+
+  if (g_utf8_validate(text.c_str(), text.size(), &invalid_data_ptr) != TRUE)
+    throw std::invalid_argument(std::string("base::reflow_text received an invalid string: ") + text);
+  
+  const std::string initial = (indent_first && use_fill) ? left_fill : "";
+  const std::string new_line = use_fill ? std::string("\n") + left_fill : std::string("\n");
+  std::string result = initial;
+
+  const char *char_string = text.c_str();
+  const char *iter = char_string;
+  const char *line_begin = char_string;
+
+  unsigned int space_position_source = 0;
+  unsigned int line_char_counter = 0;
+  unsigned int line_counter = 0;
+  unsigned int char_count_after_space = 0;
+  unsigned int text_real_length = use_fill ? line_length - left_fill_length : line_length;
+
+
+
+  while (*iter)
+  {
+    //  Get the full utf8 char into the result string
+    result += std::string(iter, g_utf8_skip[*(const guchar *)(iter)]);
+
+    line_char_counter++;
+    char_count_after_space++;
+
+    if (g_unichar_isspace(*iter) && line_char_counter > left_fill_length)
+    {
+        space_position_source = iter - char_string + 1;
+        char_count_after_space = 0;
+    }
+
+    if (line_char_counter == text_real_length)
+    {
+      //  Check for special case when we have a word as big as a line
+      if (char_count_after_space == text_real_length)
+      {
+        result += new_line;
+
+        space_position_source += char_count_after_space;
+        line_begin = (char *)char_string + space_position_source;
+        line_char_counter = char_count_after_space = 0;
+      }
+      else
+      {
+        //  Find last space character position in the result string
+        unsigned int break_position = space_position_source + line_counter * new_line.size() + initial.size();
+
+        //  Insert a \n in the right position, right after the space char(or at the end of the string)
+        result.size() == break_position ? result += new_line : result.insert(break_position, new_line);
+
+        //  Set the begining of the line of the source
+        line_begin = (char *)char_string + space_position_source;
+
+        //  Mark the characters that were already inserted after the new line
+        line_char_counter = char_count_after_space;
+      }
+      
+      if (++line_counter == max_lines)
+      {
+        result.resize(result.size() - char_count_after_space - new_line.size());
+        result += "\n(...)";
+        break;
+      }
+    }
+
+    iter = g_utf8_next_char((gchar *)iter);      //  Get the next char from the sequence
+  }
+
+#ifdef DEBUG
+  if (g_utf8_validate(result.c_str(), result.size(), &invalid_data_ptr) != TRUE)
+    throw std::logic_error(strfmt("base::reflow_text produced an invalid string:\nInput:\n%s\nOutput:\n%s", text.c_str(), result.c_str()));
+#endif
+  
+  return result;
+}
+
+} // namespace base
