@@ -51,6 +51,8 @@ using namespace base;
 CodeEditorConfig::CodeEditorConfig(SyntaxHighlighterLanguage language)
 {
   _used_language = language;
+  _language_element = NULL;
+  _document = NULL;
 
   std::string lexer;
   std::string override_lexer;
@@ -88,6 +90,10 @@ CodeEditorConfig::CodeEditorConfig(SyntaxHighlighterLanguage language)
     lexer = "SCLEX_PYTHON";
     break;
 
+  case mforms::LanguageCpp:
+    lexer = "SCLEX_CPP";
+    break;
+
   default:
     return;
   }
@@ -97,7 +103,6 @@ CodeEditorConfig::CodeEditorConfig(SyntaxHighlighterLanguage language)
   if (!g_file_test(config_file.c_str(), G_FILE_TEST_EXISTS))
     config_file = App::get()->get_resource_path("") + "/data/code_editor.xml";
 
-  _language_element = NULL;
   _document = new TiXmlDocument(config_file);
   if (!_document->LoadFile())
   {
@@ -262,6 +267,9 @@ CodeEditor::CodeEditor(void *host)
   _code_editor_impl->send_editor(this, SCI_SETCODEPAGE, SC_CP_UTF8, 0);
   _context_menu = NULL;
   _find_panel = NULL;
+  _scroll_on_resize = true;
+
+  setup();
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -274,6 +282,92 @@ CodeEditor::~CodeEditor()
 
 //--------------------------------------------------------------------------------------------------
 
+void CodeEditor::setup()
+{
+  _code_editor_impl->send_editor(this, SCI_SETLEXER, SCLEX_NULL, 0);
+  _code_editor_impl->send_editor(this, SCI_STYLERESETDEFAULT, 0, 0); // Reset default style to what it was initially.
+  _code_editor_impl->send_editor(this, SCI_STYLECLEARALL, 0, 0); // Set all other styles to the default style.
+
+  _code_editor_impl->send_editor(this, SCI_SETMARGINTYPEN, 0, SC_MARGIN_NUMBER);
+#if defined(__APPLE__)
+  _code_editor_impl->send_editor(this, SCI_STYLESETSIZE, STYLE_LINENUMBER, (sptr_t)10);
+#else
+  _code_editor_impl->send_editor(this, SCI_STYLESETSIZE, STYLE_LINENUMBER, (sptr_t)8);
+#endif
+  int lineNumberStyleWidth = _code_editor_impl->send_editor(this, SCI_TEXTWIDTH, STYLE_LINENUMBER, (long)"_999999");
+  _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 0, lineNumberStyleWidth);
+  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 0, 0);
+
+  // Margin: Markers.
+  _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 1, 16);
+  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 1, 1);
+
+  // Folder setup.
+  _code_editor_impl->send_editor(this, SCI_SETPROPERTY, (uptr_t)"fold", (sptr_t)"1");
+  _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 2, 16);
+  _code_editor_impl->send_editor(this, SCI_SETMARGINMASKN, 2, SC_MASK_FOLDERS);
+  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 2, 1); // Margin is clickable.
+  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPEN, SC_MARK_BOXMINUS);
+  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDER, SC_MARK_BOXPLUS);
+  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE);
+  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNER);
+  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDEREND, SC_MARK_BOXPLUSCONNECTED);
+  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPENMID, SC_MARK_BOXMINUSCONNECTED);
+  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNER);
+  for (int n= 25; n < 32; ++n) // Markers 25..31 are reserved for folding.
+  {
+    _code_editor_impl->send_editor(this, SCI_MARKERSETFORE, n, 0xffffff);
+    _code_editor_impl->send_editor(this, SCI_MARKERSETBACK, n, 0x404040);
+  }
+
+  // Margin: Line number style.
+  _code_editor_impl->send_editor(this, SCI_STYLESETFORE, STYLE_LINENUMBER, 0x404040);
+  _code_editor_impl->send_editor(this, SCI_STYLESETBACK, STYLE_LINENUMBER, 0xE0E0E0);
+
+  // Init markers & indicators for highlighting of syntax errors.
+  _code_editor_impl->send_editor(this, SCI_INDICSETFORE, ERROR_INDICATOR, 0x2119D0);
+  _code_editor_impl->send_editor(this, SCI_INDICSETUNDER, ERROR_INDICATOR, 1);
+  _code_editor_impl->send_editor(this, SCI_INDICSETSTYLE, ERROR_INDICATOR, INDIC_SQUIGGLE);
+
+  // Gutter markers for errors and statements, breakpoints and current code line.
+  setup_marker(CE_STATEMENT_MARKER, "editor_statement");
+  setup_marker(CE_ERROR_MARKER, "editor_error");
+  setup_marker(CE_BREAKPOINT_MARKER, "editor_breakpoint");
+  setup_marker(CE_BREAKPOINT_HIT_MARKER, "editor_breakpoint_hit");
+  setup_marker(CE_CURRENT_LINE_MARKER, "editor_current_pos");
+
+  // Other settings.
+  Color color = App::get()->get_system_color(mforms::SystemColorHighlight);
+  int rawColor = (int)(255 * color.red) + ((int)(255 * color.green) << 8) +
+    ((int)(255 * color.blue) << 16);
+  _code_editor_impl->send_editor(this, SCI_SETSELBACK, 1, rawColor);
+  _code_editor_impl->send_editor(this, SCI_SETSELFORE, 1, 0xFFFFFF);
+
+  _code_editor_impl->send_editor(this, SCI_SETCARETLINEVISIBLE, 1, 0);
+  _code_editor_impl->send_editor(this, SCI_SETCARETLINEBACK, 0xF8C800, 0);
+  _code_editor_impl->send_editor(this, SCI_SETCARETLINEBACKALPHA, 20, 0);
+
+  // - Tabulators + indentation
+  _code_editor_impl->send_editor(this, SCI_SETTABINDENTS, 1, 0);
+  _code_editor_impl->send_editor(this, SCI_SETBACKSPACEUNINDENTS, 1, 0);
+
+  // - Call tips
+  _code_editor_impl->send_editor(this, SCI_CALLTIPSETFORE, 0x202020, 0);
+  _code_editor_impl->send_editor(this, SCI_CALLTIPSETBACK, 0xF0F0F0, 0);
+
+  _code_editor_impl->send_editor(this, SCI_SETMOUSEDWELLTIME, 200, 0);
+
+  // - Line ending + scrolling.
+  _code_editor_impl->send_editor(this, SCI_SETSCROLLWIDTHTRACKING, 1, 0);
+  _code_editor_impl->send_editor(this, SCI_SETEOLMODE, SC_EOL_LF, 0);
+
+  // - Auto completion
+  _code_editor_impl->send_editor(this, SCI_AUTOCSETSEPARATOR, AC_LIST_SEPARATOR, 0);
+  _code_editor_impl->send_editor(this, SCI_AUTOCSETTYPESEPARATOR, AC_TYPE_SEPARATOR, 0);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void CodeEditor::set_text(const char* text)
 {
   _code_editor_impl->send_editor(this, SCI_SETTEXT, 0, (sptr_t)text);
@@ -562,6 +656,10 @@ void CodeEditor::set_language(SyntaxHighlighterLanguage language)
     _code_editor_impl->send_editor(this, SCI_SETLEXER, SCLEX_PYTHON, 0);
     break;
 
+  case mforms::LanguageCpp:
+    _code_editor_impl->send_editor(this, SCI_SETLEXER, SCLEX_CPP, 0);
+    break;
+
   default:
       // No language. Reset all styling so the editor appears without syntax highlighting.
     _code_editor_impl->send_editor(this, SCI_SETLEXER, SCLEX_NULL, 0);
@@ -570,87 +668,6 @@ void CodeEditor::set_language(SyntaxHighlighterLanguage language)
     return;
   }
 
-  // Number of styles needed by this lexer.
-  _code_editor_impl->send_editor(this, SCI_SETSTYLEBITS, _code_editor_impl->send_editor(this, SCI_GETSTYLEBITSNEEDED, 0, 0), 0);
-
-  _code_editor_impl->send_editor(this, SCI_SETMARGINTYPEN, 0, SC_MARGIN_NUMBER);
-#if defined(__APPLE__)
-  _code_editor_impl->send_editor(this, SCI_STYLESETSIZE, STYLE_LINENUMBER, (sptr_t)10);
-#else
-  _code_editor_impl->send_editor(this, SCI_STYLESETSIZE, STYLE_LINENUMBER, (sptr_t)8);
-#endif
-  int lineNumberStyleWidth = _code_editor_impl->send_editor(this, SCI_TEXTWIDTH, STYLE_LINENUMBER, (long)"_999999");
-  _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 0, lineNumberStyleWidth);
-  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 0, 0);
-
-  // Margin: Markers.
-  _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 1, 16);
-  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 1, 1);
-
-  // Folder setup.
-  _code_editor_impl->send_editor(this, SCI_SETPROPERTY, (uptr_t)"fold", (sptr_t)"1");
-  _code_editor_impl->send_editor(this, SCI_SETMARGINWIDTHN, 2, 16);
-  _code_editor_impl->send_editor(this, SCI_SETMARGINMASKN, 2, SC_MASK_FOLDERS);
-  _code_editor_impl->send_editor(this, SCI_SETMARGINSENSITIVEN, 2, 1); // Margin is clickable.
-  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPEN, SC_MARK_BOXMINUS);
-  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDER, SC_MARK_BOXPLUS);
-  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE);
-  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNER);
-  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDEREND, SC_MARK_BOXPLUSCONNECTED);
-  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPENMID, SC_MARK_BOXMINUSCONNECTED);
-  _code_editor_impl->send_editor(this, SCI_MARKERDEFINE, SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNER);
-  for (int n= 25; n < 32; ++n) // Markers 25..31 are reserved for folding.
-  {
-    _code_editor_impl->send_editor(this, SCI_MARKERSETFORE, n, 0xffffff);
-    _code_editor_impl->send_editor(this, SCI_MARKERSETBACK, n, 0x404040);
-  }
-
-  // Margin: Line number style.
-  _code_editor_impl->send_editor(this, SCI_STYLESETFORE, STYLE_LINENUMBER, 0x404040);
-  _code_editor_impl->send_editor(this, SCI_STYLESETBACK, STYLE_LINENUMBER, 0xE0E0E0);
-
-  // Init markers & indicators for highlighting of syntax errors.
-  _code_editor_impl->send_editor(this, SCI_INDICSETFORE, ERROR_INDICATOR, 0x2119D0);
-  _code_editor_impl->send_editor(this, SCI_INDICSETUNDER, ERROR_INDICATOR, 1);
-  _code_editor_impl->send_editor(this, SCI_INDICSETSTYLE, ERROR_INDICATOR, INDIC_SQUIGGLE);
-
-  // Gutter markers for errors and statements, breakpoints and current code line.
-  setup_marker(CE_STATEMENT_MARKER, "editor_statement");
-  setup_marker(CE_ERROR_MARKER, "editor_error");
-  setup_marker(CE_BREAKPOINT_MARKER, "editor_breakpoint");
-  setup_marker(CE_BREAKPOINT_HIT_MARKER, "editor_breakpoint_hit");
-  setup_marker(CE_CURRENT_LINE_MARKER, "editor_current_pos");
-
-  // Other settings.
-  Color color = App::get()->get_system_color(mforms::SystemColorHighlight);
-  int rawColor = (int)(255 * color.red) + ((int)(255 * color.green) << 8) +
-    ((int)(255 * color.blue) << 16);
-  _code_editor_impl->send_editor(this, SCI_SETSELBACK, 1, rawColor);
-  _code_editor_impl->send_editor(this, SCI_SETSELFORE, 1, 0xFFFFFF);
-
-  _code_editor_impl->send_editor(this, SCI_SETCARETLINEVISIBLE, 1, 0);
-  _code_editor_impl->send_editor(this, SCI_SETCARETLINEBACK, 0xF8C800, 0);
-  _code_editor_impl->send_editor(this, SCI_SETCARETLINEBACKALPHA, 20, 0);
-
-  // - Tabulators + indentation
-  _code_editor_impl->send_editor(this, SCI_SETTABINDENTS, 1, 0);
-  _code_editor_impl->send_editor(this, SCI_SETBACKSPACEUNINDENTS, 1, 0);
-
-  // - Call tips
-  _code_editor_impl->send_editor(this, SCI_CALLTIPSETFORE, 0x202020, 0);
-  _code_editor_impl->send_editor(this, SCI_CALLTIPSETBACK, 0xF0F0F0, 0);
-
-  _code_editor_impl->send_editor(this, SCI_SETMOUSEDWELLTIME, 200, 0);
-
-  // - Line ending + scrolling.
-  _code_editor_impl->send_editor(this, SCI_SETSCROLLWIDTHTRACKING, 1, 0);
-  _code_editor_impl->send_editor(this, SCI_SETEOLMODE, SC_EOL_LF, 0);
-  
-  // - Auto completion
-  _code_editor_impl->send_editor(this, SCI_AUTOCSETSEPARATOR, AC_LIST_SEPARATOR, 0);
-  _code_editor_impl->send_editor(this, SCI_AUTOCSETTYPESEPARATOR, AC_TYPE_SEPARATOR, 0);
-  
-  // After the general setup load language settings/keywords etc. from config file.
   load_configuration(language);
 }
 
@@ -994,6 +1011,12 @@ void CodeEditor::set_features(CodeEditorFeature features, bool flag)
 
   if ((features & mforms::FeatureConvertEolOnPaste) != 0)
     _code_editor_impl->send_editor(this, SCI_SETPASTECONVERTENDINGS, flag, 0);
+
+  if ((features & mforms::FeatureScrollOnResize) != 0)
+    _scroll_on_resize = true;
+
+  if ((features & mforms::FeatureFolding) != 0)
+    _code_editor_impl->send_editor(this, SCI_SETPROPERTY, (uptr_t)"fold", flag ? (sptr_t)"1" : (sptr_t)"0");
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1030,6 +1053,15 @@ void CodeEditor::toggle_features(CodeEditorFeature features)
   if ((features & mforms::FeatureConvertEolOnPaste) != 0)
     set_features(mforms::FeatureConvertEolOnPaste,
       _code_editor_impl->send_editor(this, SCI_GETPASTECONVERTENDINGS, 0, 0) != 0);
+
+  if ((features & mforms::FeatureScrollOnResize) != 0)
+    _scroll_on_resize = !_scroll_on_resize;
+
+  if ((features & mforms::FeatureFolding) != 0)
+  {
+    bool folding_enabled = _code_editor_impl->send_editor(this, SCI_GETPROPERTYINT, (uptr_t)"fold", 0) != 0;
+    _code_editor_impl->send_editor(this, SCI_SETPROPERTY, (uptr_t)"fold", folding_enabled ? (sptr_t)"0" : (sptr_t)"1");
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1172,13 +1204,6 @@ void CodeEditor::set_status_text(const std::string& text)
 
 //--------------------------------------------------------------------------------------------------
 
-void CodeEditor::enable_folding(bool enable)
-{
-  _code_editor_impl->send_editor(this, SCI_SETPROPERTY, (uptr_t)"fold", enable ? (sptr_t)"1" : (sptr_t)"0");
-}
-
-//--------------------------------------------------------------------------------------------------
-
 void CodeEditor::show_find_panel(bool replace)
 {
   if (_find_panel == NULL)
@@ -1575,8 +1600,17 @@ sptr_t CodeEditor::send_editor(unsigned int message, uptr_t wParam, sptr_t lPara
 
 //--------------------------------------------------------------------------------------------------
 
-
 void CodeEditor::lost_focus()
 {
   _signal_lost_focus();
 }
+
+//--------------------------------------------------------------------------------------------------
+
+void CodeEditor::resize()
+{
+  if (_scroll_on_resize)
+    _code_editor_impl->send_editor(this, SCI_SCROLLCARET, 0, 0);
+}
+
+//--------------------------------------------------------------------------------------------------
